@@ -13,9 +13,9 @@ pub mod vecx;
 use crate::merkle::{MerkleTree, MerkleTreeStore, Proof, ProofEntry};
 use crate::{
     basic::{
-        mapx::Mapx, mapx_ord::MapxOrd, mapx_ord_rawkey::MapxOrdRk,
-        mapx_ord_rawvalue::MapxOrdRv, mapx_raw::MapxRaw, orphan::Orphan, vecx::Vecx,
-        vecx_raw::VecxRaw,
+        mapx::Mapx, mapx_ord::MapxOrd, mapx_ord_rawkey::MapxOrdRawKey,
+        mapx_ord_rawvalue::MapxOrdRawValue, mapx_raw::MapxRaw, orphan::Orphan,
+        vecx::Vecx, vecx_raw::VecxRaw,
     },
     BranchName, ParentBranchName, VersionName,
 };
@@ -54,16 +54,6 @@ pub trait VsMgmt {
         branch_name: BranchName,
     ) -> bool;
 
-    /// Check if a version is directly created on the default branch.
-    fn version_created(&self, version_name: VersionName) -> bool;
-
-    /// Check if a version is directly created on a specified branch(exclude its parents).
-    fn version_created_on_branch(
-        &self,
-        version_name: VersionName,
-        branch_name: BranchName,
-    ) -> bool;
-
     /// Remove the newest version on the default branch.
     ///
     /// 'Write'-like operations on branches and versions are different from operations on data.
@@ -83,22 +73,39 @@ pub trait VsMgmt {
     fn version_pop_by_branch(&self, branch_name: BranchName) -> Result<()>;
 
     /// Merge all changes made by new versions after the base version into the base version.
-    fn version_rebase(&self, base_version: VersionName) -> Result<()>;
+    ///
+    /// # Safety
+    ///
+    /// It's the caller's duty to ensure that
+    /// the `base_version` was created directly by the `branch_id`,
+    /// or the data records of other branches may be corrupted.
+    unsafe fn version_rebase(&self, base_version: VersionName) -> Result<()>;
 
     /// Merge all changes made by new versions after the base version into the base version.
-    fn version_rebase_by_branch(
+    ///
+    /// # Safety
+    ///
+    /// It's the caller's duty to ensure that
+    /// the `base_version` was created directly by the `branch_id`,
+    /// or the data records of other branches may be corrupted.
+    unsafe fn version_rebase_by_branch(
         &self,
         base_version: VersionName,
         branch_name: BranchName,
     ) -> Result<()>;
 
     /// Create a new branch based on the head of the default branch.
-    fn branch_create(&self, branch_name: BranchName) -> Result<()>;
+    fn branch_create(
+        &self,
+        branch_name: BranchName,
+        version_name: VersionName,
+    ) -> Result<()>;
 
     /// Create a new branch based on the head of a specified branch.
     fn branch_create_by_base_branch(
         &self,
         branch_name: BranchName,
+        version_name: VersionName,
         base_branch_name: ParentBranchName,
     ) -> Result<()>;
 
@@ -106,6 +113,7 @@ pub trait VsMgmt {
     fn branch_create_by_base_branch_version(
         &self,
         branch_name: BranchName,
+        version_name: VersionName,
         base_branch_name: ParentBranchName,
         base_version_name: VersionName,
     ) -> Result<()>;
@@ -156,13 +164,25 @@ pub trait VsMgmt {
     /// and should not do any tracing.
     fn branch_pop_version(&self, branch_name: BranchName) -> Result<()>;
 
-    /// Merge a branch to its parent branch.
-    ///
-    /// NOTE: the original branch will be deleted.
-    fn branch_merge_to_parent(&self, branch_name: BranchName) -> Result<()>;
+    /// Merge a branch into another.
+    fn branch_merge_to(
+        &self,
+        branch_name: BranchName,
+        target_branch_name: BranchName,
+    ) -> Result<()>;
 
-    /// Check if a branch has children branches.
-    fn branch_has_children(&self, branch_name: BranchName) -> bool;
+    /// Merge a branch into another,
+    /// even if new different versions have been created on the target branch.
+    ///
+    /// # Safety
+    ///
+    /// If new different versions have been created on the target branch,
+    /// the data records referenced by other branches may be corrupted.
+    unsafe fn branch_merge_to_force(
+        &self,
+        branch_name: BranchName,
+        target_branch_name: BranchName,
+    ) -> Result<()>;
 
     /// Make a branch to be default,
     /// all default operations will be applied to it.
@@ -170,13 +190,6 @@ pub trait VsMgmt {
 
     /// Clean outdated versions out of the default branch.
     fn prune(&self, reserved_ver_num: Option<usize>) -> Result<()>;
-
-    /// Clean outdated versions out of a specified branch.
-    fn prune_by_branch(
-        &self,
-        branch_name: BranchName,
-        reserved_ver_num: Option<usize>,
-    ) -> Result<()>;
 }
 
 #[macro_export(super)]
@@ -218,23 +231,6 @@ macro_rules! impl_vs_methods {
                 .version_exists_on_branch(version_name, branch_name)
         }
 
-        /// Check if a version is directly created on the default branch.
-        #[inline(always)]
-        fn version_created(&self, version_name: VersionName) -> bool {
-            self.inner.version_created(version_name)
-        }
-
-        /// Check if a version is directly created on a specified branch(exclude its parents).
-        #[inline(always)]
-        fn version_created_on_branch(
-            &self,
-            version_name: VersionName,
-            branch_name: BranchName,
-        ) -> bool {
-            self.inner
-                .version_created_on_branch(version_name, branch_name)
-        }
-
         /// Remove the newest version on the default branch.
         ///
         /// 'Write'-like operations on branches and versions are different from operations on data.
@@ -260,14 +256,26 @@ macro_rules! impl_vs_methods {
         }
 
         /// Merge all changes made by new versions after the base version into the base version.
+        ///
+        /// # Safety
+        ///
+        /// It's the caller's duty to ensure that
+        /// the `base_version` was created directly by the `branch_id`,
+        /// or the data records of other branches may be corrupted.
         #[inline(always)]
-        fn version_rebase(&self, base_version: VersionName) -> Result<()> {
+        unsafe fn version_rebase(&self, base_version: VersionName) -> Result<()> {
             self.inner.version_rebase(base_version).c(d!())
         }
 
         /// Merge all changes made by new versions after the base version into the base version.
+        ///
+        /// # Safety
+        ///
+        /// It's the caller's duty to ensure that
+        /// the `base_version` was created directly by the `branch_id`,
+        /// or the data records of other branches may be corrupted.
         #[inline(always)]
-        fn version_rebase_by_branch(
+        unsafe fn version_rebase_by_branch(
             &self,
             base_version: VersionName,
             branch_name: BranchName,
@@ -279,8 +287,12 @@ macro_rules! impl_vs_methods {
 
         /// Create a new branch based on the head of the default branch.
         #[inline(always)]
-        fn branch_create(&self, branch_name: BranchName) -> Result<()> {
-            self.inner.branch_create(branch_name).c(d!())
+        fn branch_create(
+            &self,
+            branch_name: BranchName,
+            version_name: VersionName,
+        ) -> Result<()> {
+            self.inner.branch_create(branch_name, version_name).c(d!())
         }
 
         /// Create a new branch based on the head of a specified branch.
@@ -288,10 +300,15 @@ macro_rules! impl_vs_methods {
         fn branch_create_by_base_branch(
             &self,
             branch_name: BranchName,
+            version_name: VersionName,
             base_branch_name: ParentBranchName,
         ) -> Result<()> {
             self.inner
-                .branch_create_by_base_branch(branch_name, base_branch_name)
+                .branch_create_by_base_branch(
+                    branch_name,
+                    version_name,
+                    base_branch_name,
+                )
                 .c(d!())
         }
 
@@ -300,12 +317,14 @@ macro_rules! impl_vs_methods {
         fn branch_create_by_base_branch_version(
             &self,
             branch_name: BranchName,
+            version_name: VersionName,
             base_branch_name: ParentBranchName,
             base_version_name: VersionName,
         ) -> Result<()> {
             self.inner
                 .branch_create_by_base_branch_version(
                     branch_name,
+                    version_name,
                     base_branch_name,
                     base_version_name,
                 )
@@ -377,16 +396,33 @@ macro_rules! impl_vs_methods {
             self.inner.branch_pop_version(branch_name).c(d!())
         }
 
-        /// Merge a branch to its parent branch.
+        /// Merge a branch into another
         #[inline(always)]
-        fn branch_merge_to_parent(&self, branch_name: BranchName) -> Result<()> {
-            self.inner.branch_merge_to_parent(branch_name).c(d!())
+        fn branch_merge_to(
+            &self,
+            branch_name: BranchName,
+            target_branch_name: BranchName,
+        ) -> Result<()> {
+            self.inner
+                .branch_merge_to(branch_name, target_branch_name)
+                .c(d!())
         }
 
-        /// Check if a branch has children branches.
-        #[inline(always)]
-        fn branch_has_children(&self, branch_name: BranchName) -> bool {
-            self.inner.branch_has_children(branch_name)
+        /// Merge a branch into another,
+        /// even if new different versions have been created on the target branch.
+        ///
+        /// # Safety
+        ///
+        /// If new different versions have been created on the target branch,
+        /// the data records referenced by other branches may be corrupted.
+        unsafe fn branch_merge_to_force(
+            &self,
+            branch_name: BranchName,
+            target_branch_name: BranchName,
+        ) -> Result<()> {
+            self.inner
+                .branch_merge_to_force(branch_name, target_branch_name)
+                .c(d!())
         }
 
         /// Make a branch to be default,
@@ -401,18 +437,6 @@ macro_rules! impl_vs_methods {
         fn prune(&self, reserved_ver_num: Option<usize>) -> Result<()> {
             self.inner.prune(reserved_ver_num).c(d!())
         }
-
-        /// Clean outdated versions out of a specified reserved number.
-        #[inline(always)]
-        fn prune_by_branch(
-            &self,
-            branch_name: BranchName,
-            reserved_ver_num: Option<usize>,
-        ) -> Result<()> {
-            self.inner
-                .prune_by_branch(branch_name, reserved_ver_num)
-                .c(d!())
-        }
     };
 }
 
@@ -445,16 +469,6 @@ macro_rules! impl_vs_methods_nope {
             true
         }
 
-        #[inline(always)]
-        fn version_created(&self, _: VersionName) -> bool {
-            true
-        }
-
-        #[inline(always)]
-        fn version_created_on_branch(&self, _: VersionName, _: BranchName) -> bool {
-            true
-        }
-
         #[inline(always)]
         fn version_pop(&self) -> Result<()> {
             Ok(())
@@ -466,17 +480,21 @@ macro_rules! impl_vs_methods_nope {
         }
 
         #[inline(always)]
-        fn version_rebase(&self, _: VersionName) -> Result<()> {
+        unsafe fn version_rebase(&self, _: VersionName) -> Result<()> {
             Ok(())
         }
 
         #[inline(always)]
-        fn version_rebase_by_branch(&self, _: VersionName, _: BranchName) -> Result<()> {
+        unsafe fn version_rebase_by_branch(
+            &self,
+            _: VersionName,
+            _: BranchName,
+        ) -> Result<()> {
             Ok(())
         }
 
         #[inline(always)]
-        fn branch_create(&self, _: BranchName) -> Result<()> {
+        fn branch_create(&self, _: BranchName, _: VersionName) -> Result<()> {
             Ok(())
         }
 
@@ -484,6 +502,7 @@ macro_rules! impl_vs_methods_nope {
         fn branch_create_by_base_branch(
             &self,
             _: BranchName,
+            _: VersionName,
             _: ParentBranchName,
         ) -> Result<()> {
             Ok(())
@@ -493,6 +512,7 @@ macro_rules! impl_vs_methods_nope {
         fn branch_create_by_base_branch_version(
             &self,
             _: BranchName,
+            _: VersionName,
             _: ParentBranchName,
             _: VersionName,
         ) -> Result<()> {
@@ -529,13 +549,16 @@ macro_rules! impl_vs_methods_nope {
         }
 
         #[inline(always)]
-        fn branch_merge_to_parent(&self, _: BranchName) -> Result<()> {
+        fn branch_merge_to(&self, _: BranchName, _: BranchName) -> Result<()> {
             Ok(())
         }
 
-        #[inline(always)]
-        fn branch_has_children(&self, _: BranchName) -> bool {
-            true
+        unsafe fn branch_merge_to_force(
+            &self,
+            _: BranchName,
+            _: BranchName,
+        ) -> Result<()> {
+            Ok(())
         }
 
         #[inline(always)]
@@ -547,11 +570,6 @@ macro_rules! impl_vs_methods_nope {
         fn prune(&self, _: Option<usize>) -> Result<()> {
             Ok(())
         }
-
-        #[inline(always)]
-        fn prune_by_branch(&self, _: BranchName, __: Option<usize>) -> Result<()> {
-            Ok(())
-        }
     };
 }
 
@@ -567,11 +585,11 @@ impl<K, V> VsMgmt for MapxOrd<K, V> {
     impl_vs_methods_nope!();
 }
 
-impl<V> VsMgmt for MapxOrdRk<V> {
+impl<V> VsMgmt for MapxOrdRawKey<V> {
     impl_vs_methods_nope!();
 }
 
-impl<K> VsMgmt for MapxOrdRv<K> {
+impl<K> VsMgmt for MapxOrdRawValue<K> {
     impl_vs_methods_nope!();
 }
 
@@ -729,26 +747,6 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
         true
     }
 
-    #[inline(always)]
-    fn version_created(&self, version_name: VersionName) -> bool {
-        if let Some(i) = self.as_ref() {
-            return i.version_created(version_name);
-        }
-        true
-    }
-
-    #[inline(always)]
-    fn version_created_on_branch(
-        &self,
-        version_name: VersionName,
-        branch_name: BranchName,
-    ) -> bool {
-        if let Some(i) = self.as_ref() {
-            return i.version_created_on_branch(version_name, branch_name);
-        }
-        true
-    }
-
     #[inline(always)]
     fn version_pop(&self) -> Result<()> {
         if let Some(i) = self.as_ref() {
@@ -766,7 +764,7 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
     }
 
     #[inline(always)]
-    fn version_rebase(&self, base_version: VersionName) -> Result<()> {
+    unsafe fn version_rebase(&self, base_version: VersionName) -> Result<()> {
         if let Some(i) = self.as_ref() {
             i.version_rebase(base_version).c(d!())?;
         }
@@ -774,7 +772,7 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
     }
 
     #[inline(always)]
-    fn version_rebase_by_branch(
+    unsafe fn version_rebase_by_branch(
         &self,
         base_version: VersionName,
         branch_name: BranchName,
@@ -787,9 +785,13 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
     }
 
     #[inline(always)]
-    fn branch_create(&self, branch_name: BranchName) -> Result<()> {
+    fn branch_create(
+        &self,
+        branch_name: BranchName,
+        version_name: VersionName,
+    ) -> Result<()> {
         if let Some(i) = self.as_ref() {
-            i.branch_create(branch_name).c(d!())?;
+            i.branch_create(branch_name, version_name).c(d!())?;
         }
         Ok(())
     }
@@ -798,10 +800,11 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
     fn branch_create_by_base_branch(
         &self,
         branch_name: BranchName,
+        version_name: VersionName,
         base_branch_name: ParentBranchName,
     ) -> Result<()> {
         if let Some(i) = self.as_ref() {
-            i.branch_create_by_base_branch(branch_name, base_branch_name)
+            i.branch_create_by_base_branch(branch_name, version_name, base_branch_name)
                 .c(d!())?;
         }
         Ok(())
@@ -811,12 +814,14 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
     fn branch_create_by_base_branch_version(
         &self,
         branch_name: BranchName,
+        version_name: VersionName,
         base_branch_name: ParentBranchName,
         base_version_name: VersionName,
     ) -> Result<()> {
         if let Some(i) = self.as_ref() {
             i.branch_create_by_base_branch_version(
                 branch_name,
+                version_name,
                 base_branch_name,
                 base_version_name,
             )
@@ -879,19 +884,28 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
     }
 
     #[inline(always)]
-    fn branch_merge_to_parent(&self, branch_name: BranchName) -> Result<()> {
+    fn branch_merge_to(
+        &self,
+        branch_name: BranchName,
+        target_branch_name: BranchName,
+    ) -> Result<()> {
         if let Some(i) = self.as_ref() {
-            i.branch_merge_to_parent(branch_name).c(d!())?;
+            i.branch_merge_to(branch_name, target_branch_name).c(d!())?;
         }
         Ok(())
     }
 
     #[inline(always)]
-    fn branch_has_children(&self, branch_name: BranchName) -> bool {
+    unsafe fn branch_merge_to_force(
+        &self,
+        branch_name: BranchName,
+        target_branch_name: BranchName,
+    ) -> Result<()> {
         if let Some(i) = self.as_ref() {
-            return i.branch_has_children(branch_name);
+            i.branch_merge_to_force(branch_name, target_branch_name)
+                .c(d!())?;
         }
-        true
+        Ok(())
     }
 
     #[inline(always)]
@@ -909,16 +923,4 @@ impl<T: VsMgmt> VsMgmt for Option<T> {
         }
         Ok(())
     }
-
-    #[inline(always)]
-    fn prune_by_branch(
-        &self,
-        branch_name: BranchName,
-        reserved_ver_num: Option<usize>,
-    ) -> Result<()> {
-        if let Some(i) = self.as_ref() {
-            i.prune_by_branch(branch_name, reserved_ver_num).c(d!())?;
-        }
-        Ok(())
-    }
 }
